@@ -15,6 +15,7 @@ class CrushTester {
 
   map<int, int> device_weight;
   int min_rule, max_rule;
+  int ruleset;
   int min_x, max_x;
   int min_rep, max_rep;
 
@@ -168,6 +169,7 @@ public:
   CrushTester(CrushWrapper& c, ostream& eo)
     : crush(c), err(eo),
       min_rule(-1), max_rule(-1),
+      ruleset(-1),
       min_x(-1), max_x(-1),
       min_rep(-1), max_rep(-1),
       num_batches(1),
@@ -333,6 +335,10 @@ public:
     min_rule = max_rule = rule;
   }
 
+  void set_ruleset(int rs) {
+    ruleset = rs;
+  }
+
   /**
    * check if any bucket/nodes is referencing an unknown name or type
    * @param max_id rejects any non-bucket items with id less than this number,
@@ -344,7 +350,8 @@ public:
   int test();
   int test_with_crushtool(const string& crushtool,
 			  int max_id = -1,
-			  int timeout = 0);
+			  int timeout = 0,
+			  int ruleset = -1);
 };
 
 #endif
