@@ -25,13 +25,13 @@
 #include <set>
 #include <map>
 #include <string>
-using namespace std;
 
 #include "common/config.h"
 
 #include "include/CompatSet.h"
 #include "include/ceph_features.h"
 #include "common/Formatter.h"
+#include "mds/mdstypes.h"
 
 /*
 
@@ -69,58 +69,79 @@ extern CompatSet get_mdsmap_compat_set_base(); // pre v0.20
 #define MDS_FEATURE_INCOMPAT_ENCODING CompatSet::Feature(5, "mds uses versioned encoding")
 #define MDS_FEATURE_INCOMPAT_OMAPDIRFRAG CompatSet::Feature(6, "dirfrag is stored in omap")
 #define MDS_FEATURE_INCOMPAT_INLINE CompatSet::Feature(7, "mds uses inline data")
+#define MDS_FEATURE_INCOMPAT_NOANCHOR CompatSet::Feature(8, "no anchor table")
+
+#define MDS_FS_NAME_DEFAULT "cephfs"
 
 class MDSMap {
 public:
-  // mds states
-  /*
-  static const int STATE_DNE =        CEPH_MDS_STATE_DNE;  // down, never existed.
-  static const int STATE_DESTROYING = CEPH_MDS_STATE_DESTROYING;  // down, existing, semi-destroyed.
-  static const int STATE_FAILED =     CEPH_MDS_STATE_FAILED;  // down, active subtrees; needs to be recovered.
-  */
-  static const int STATE_STOPPED =    CEPH_MDS_STATE_STOPPED;  // down, once existed, but no subtrees. empty log.
-  static const int STATE_BOOT     =   CEPH_MDS_STATE_BOOT;  // up, boot announcement.  destiny unknown.
+  /* These states are the union of the set of possible states of an MDS daemon,
+   * and the set of possible states of an MDS rank */
+  typedef enum {
+    // States of an MDS daemon not currently holding a rank
+    // ====================================================
+    STATE_NULL     =   0,                                  // null value for fns returning this type.
+    STATE_BOOT     =   CEPH_MDS_STATE_BOOT,                // up, boot announcement.  destiny unknown.
+    STATE_STANDBY  =   CEPH_MDS_STATE_STANDBY,             // up, idle.  waiting for assignment by monitor.
+    STATE_STANDBY_REPLAY = CEPH_MDS_STATE_STANDBY_REPLAY,  // up, replaying active node, ready to take over.
 
-  static const int STATE_STANDBY  =   CEPH_MDS_STATE_STANDBY;  // up, idle.  waiting for assignment by monitor.
-  static const int STATE_STANDBY_REPLAY = CEPH_MDS_STATE_STANDBY_REPLAY;  // up, replaying active node; ready to take over.
-  static const int STATE_ONESHOT_REPLAY = CEPH_MDS_STATE_REPLAYONCE; //up, replaying active node journal to verify it, then shutting down
+    // States of an MDS rank, and of any MDS daemon holding that rank
+    // ==============================================================
+    STATE_STOPPED  =   CEPH_MDS_STATE_STOPPED,        // down, once existed, but no subtrees. empty log.  may not be held by a daemon.
+    STATE_ONESHOT_REPLAY = CEPH_MDS_STATE_REPLAYONCE, // up, replaying active node journal to verify it, then shutting down
 
-  static const int STATE_CREATING  =  CEPH_MDS_STATE_CREATING;  // up, creating MDS instance (new journal, idalloc..).
-  static const int STATE_STARTING  =  CEPH_MDS_STATE_STARTING;  // up, starting prior stopped MDS instance.
+    STATE_CREATING  =  CEPH_MDS_STATE_CREATING,       // up, creating MDS instance (new journal, idalloc..).
+    STATE_STARTING  =  CEPH_MDS_STATE_STARTING,       // up, starting prior stopped MDS instance.
 
-  static const int STATE_REPLAY    =  CEPH_MDS_STATE_REPLAY;  // up, starting prior failed instance. scanning journal.
-  static const int STATE_RESOLVE   =  CEPH_MDS_STATE_RESOLVE;  // up, disambiguating distributed operations (import, rename, etc.)
-  static const int STATE_RECONNECT =  CEPH_MDS_STATE_RECONNECT;  // up, reconnect to clients
-  static const int STATE_REJOIN    =  CEPH_MDS_STATE_REJOIN; // up, replayed journal, rejoining distributed cache
-  static const int STATE_CLIENTREPLAY = CEPH_MDS_STATE_CLIENTREPLAY; // up, active
-  static const int STATE_ACTIVE =     CEPH_MDS_STATE_ACTIVE; // up, active
-  static const int STATE_STOPPING  =  CEPH_MDS_STATE_STOPPING; // up, exporting metadata (-> standby or out)
+    STATE_REPLAY    =  CEPH_MDS_STATE_REPLAY,         // up, starting prior failed instance. scanning journal.
+    STATE_RESOLVE   =  CEPH_MDS_STATE_RESOLVE,        // up, disambiguating distributed operations (import, rename, etc.)
+    STATE_RECONNECT =  CEPH_MDS_STATE_RECONNECT,      // up, reconnect to clients
+    STATE_REJOIN    =  CEPH_MDS_STATE_REJOIN,         // up, replayed journal, rejoining distributed cache
+    STATE_CLIENTREPLAY = CEPH_MDS_STATE_CLIENTREPLAY, // up, active
+    STATE_ACTIVE =     CEPH_MDS_STATE_ACTIVE,         // up, active
+    STATE_STOPPING  =  CEPH_MDS_STATE_STOPPING,       // up, exporting metadata (-> standby or out)
+    STATE_DNE       =  CEPH_MDS_STATE_DNE             // down, rank does not exist
+
+    /*
+     * In addition to explicit states, an MDS rank implicitly in state:
+     *  - STOPPED if it is not currently associated with an MDS daemon gid but it
+     *    is in MDSMap::stopped
+     *  - FAILED if it is not currently associated with an MDS daemon gid but it
+     *    is in MDSMap::failed
+     *  - DNE if it is not currently associated with an MDS daemon gid and it is
+     *    missing from both MDSMap::failed and MDSMap::stopped
+     */
+  } DaemonState;
 
   // indicate startup standby preferences for MDS
   // of course, if they have a specific rank to follow, they just set that!
-  static const int MDS_NO_STANDBY_PREF = -1; // doesn't have instructions to do anything
-  static const int MDS_STANDBY_ANY = -2; // is instructed to be standby-replay, may
-                                     // or may not have specific name to follow
-  static const int MDS_STANDBY_NAME = -3; // standby for a named MDS
-  static const int MDS_MATCHED_ACTIVE = -4; // has a matched standby, which if up
-                                            // it should follow, but otherwise should
-                                            // be assigned a rank
+  static const mds_rank_t MDS_NO_STANDBY_PREF; // doesn't have instructions to do anything
+  static const mds_rank_t MDS_STANDBY_ANY;     // is instructed to be standby-replay, may
+                                               // or may not have specific name to follow
+  static const mds_rank_t MDS_STANDBY_NAME;    // standby for a named MDS
+  static const mds_rank_t MDS_MATCHED_ACTIVE;  // has a matched standby, which if up
+                                               // it should follow, but otherwise should
+                                               // be assigned a rank
 
   struct mds_info_t {
-    uint64_t global_id;
-    string name;
-    int32_t rank;
+    mds_gid_t global_id;
+    std::string name;
+    mds_rank_t rank;
     int32_t inc;
-    int32_t state;
+    MDSMap::DaemonState state;
     version_t state_seq;
     entity_addr_t addr;
     utime_t laggy_since;
-    int32_t standby_for_rank;
-    string standby_for_name;
-    set<int32_t> export_targets;
+    mds_rank_t standby_for_rank;
+    std::string standby_for_name;
+    std::set<mds_rank_t> export_targets;
 
-    mds_info_t() : global_id(0), rank(-1), inc(0), state(STATE_STANDBY), state_seq(0),
+#if 1
+    mds_info_t() : global_id(MDS_GID_NONE), rank(MDS_RANK_NONE), inc(0), state(STATE_STANDBY), state_seq(0),
 		   standby_for_rank(MDS_NO_STANDBY_PREF) { }
+#else
+    mds_info_t();
+#endif
 
     bool laggy() const { return !(laggy_since == utime_t()); }
     void clear_laggy() { laggy_since = utime_t(); }
@@ -143,20 +164,22 @@ public:
 protected:
   // base map
   epoch_t epoch;
+  bool enabled;
+  std::string fs_name;
   uint32_t flags;        // flags
   epoch_t last_failure;  // mds epoch of last failure
   epoch_t last_failure_osd_epoch; // osd epoch of last failure; any mds entering replay needs
                                   // at least this osdmap to ensure the blacklist propagates.
   utime_t created, modified;
 
-  int32_t tableserver;   // which MDS has anchortable, snaptable
-  int32_t root;          // which MDS has root directory
+  mds_rank_t tableserver;   // which MDS has snaptable
+  mds_rank_t root;          // which MDS has root directory
 
   __u32 session_timeout;
   __u32 session_autoclose;
   uint64_t max_file_size;
 
-  set<int64_t> data_pools;  // file data pools available to clients (via an ioctl).  first is the default.
+  std::set<int64_t> data_pools;  // file data pools available to clients (via an ioctl).  first is the default.
   int64_t cas_pool;            // where CAS objects go
   int64_t metadata_pool;       // where fs metadata objects go
   
@@ -170,13 +193,13 @@ protected:
    *    @up + @failed = @in.  @in * @stopped = {}.
    */
 
-  uint32_t max_mds; /* The maximum number of active MDSes. Also, the maximum rank. */
+  mds_rank_t max_mds; /* The maximum number of active MDSes. Also, the maximum rank. */
 
-  set<int32_t> in;              // currently defined cluster
-  map<int32_t,int32_t> inc;     // most recent incarnation.
-  set<int32_t> failed, stopped; // which roles are failed or stopped
-  map<int32_t,uint64_t> up;        // who is in those roles
-  map<uint64_t,mds_info_t> mds_info;
+  std::set<mds_rank_t> in;              // currently defined cluster
+  std::map<mds_rank_t,int32_t> inc;     // most recent incarnation.
+  std::set<mds_rank_t> failed, stopped; // which roles are failed or stopped
+  std::map<mds_rank_t, mds_gid_t> up;        // who is in those roles
+  std::map<mds_gid_t, mds_info_t> mds_info;
 
   bool ever_allowed_snaps; //< the cluster has ever allowed snap creation
   bool explicitly_allowed_snaps; //< the user has explicitly enabled snap creation
@@ -190,7 +213,10 @@ public:
 
 public:
   MDSMap() 
-    : epoch(0), flags(0), last_failure(0), last_failure_osd_epoch(0), tableserver(0), root(0),
+    : epoch(0), enabled(false), fs_name(MDS_FS_NAME_DEFAULT),
+      flags(0), last_failure(0),
+      last_failure_osd_epoch(0),
+      tableserver(0), root(0),
       session_timeout(0),
       session_autoclose(0),
       max_file_size(0),
@@ -226,6 +252,8 @@ public:
   epoch_t get_epoch() const { return epoch; }
   void inc_epoch() { epoch++; }
 
+  bool get_enabled() const { return enabled; }
+
   const utime_t& get_created() const { return created; }
   void set_created(utime_t ct) { modified = created = ct; }
   const utime_t& get_modified() const { return modified; }
@@ -234,13 +262,13 @@ public:
   epoch_t get_last_failure() const { return last_failure; }
   epoch_t get_last_failure_osd_epoch() const { return last_failure_osd_epoch; }
 
-  unsigned get_max_mds() const { return max_mds; }
-  void set_max_mds(int m) { max_mds = m; }
+  mds_rank_t get_max_mds() const { return max_mds; }
+  void set_max_mds(mds_rank_t m) { max_mds = m; }
 
-  int get_tableserver() const { return tableserver; }
-  int get_root() const { return root; }
+  mds_rank_t get_tableserver() const { return tableserver; }
+  mds_rank_t get_root() const { return root; }
 
-  const set<int64_t> &get_data_pools() const { return data_pools; }
+  const std::set<int64_t> &get_data_pools() const { return data_pools; }
   int64_t get_first_data_pool() const { return *data_pools.begin(); }
   int64_t get_cas_pool() const { return cas_pool; }
   int64_t get_metadata_pool() const { return metadata_pool; }
@@ -248,24 +276,28 @@ public:
     return data_pools.count(poolid);
   }
 
-  const map<uint64_t,mds_info_t>& get_mds_info() { return mds_info; }
-  const mds_info_t& get_mds_info_gid(uint64_t gid) {
+  bool pool_in_use(int64_t poolid) const {
+    return get_enabled() && (is_data_pool(poolid) || metadata_pool == poolid);
+  }
+
+  const std::map<mds_gid_t,mds_info_t>& get_mds_info() { return mds_info; }
+  const mds_info_t& get_mds_info_gid(mds_gid_t gid) {
     assert(mds_info.count(gid));
     return mds_info[gid];
   }
-  const mds_info_t& get_mds_info(int m) {
+  const mds_info_t& get_mds_info(mds_rank_t m) {
     assert(up.count(m) && mds_info.count(up[m]));
     return mds_info[up[m]];
   }
-  uint64_t find_mds_gid_by_name(const string& s) {
-    for (map<uint64_t,mds_info_t>::const_iterator p = mds_info.begin();
+  mds_gid_t find_mds_gid_by_name(const std::string& s) {
+    for (std::map<mds_gid_t,mds_info_t>::const_iterator p = mds_info.begin();
 	 p != mds_info.end();
 	 ++p) {
       if (p->second.name == s) {
 	return p->first;
       }
     }
-    return 0;
+    return MDS_GID_NONE;
   }
 
   // counts
@@ -280,7 +312,7 @@ public:
   }
   unsigned get_num_mds(int state) const {
     unsigned n = 0;
-    for (map<uint64_t,mds_info_t>::const_iterator p = mds_info.begin();
+    for (std::map<mds_gid_t,mds_info_t>::const_iterator p = mds_info.begin();
 	 p != mds_info.end();
 	 ++p)
       if (p->second.state == state) ++n;
@@ -292,7 +324,7 @@ public:
     data_pools.insert(poolid);
   }
   int remove_data_pool(int64_t poolid) {
-    set<int64_t>::iterator p = data_pools.find(poolid);
+    std::set<int64_t>::iterator p = data_pools.find(poolid);
     if (p == data_pools.end())
       return -ENOENT;
     data_pools.erase(p);
@@ -300,45 +332,45 @@ public:
   }
 
   // sets
-  void get_mds_set(set<int>& s) {
+  void get_mds_set(std::set<mds_rank_t>& s) {
     s = in;
   }
-  void get_up_mds_set(set<int>& s) {
-    for (map<int32_t,uint64_t>::const_iterator p = up.begin();
+  void get_up_mds_set(std::set<mds_rank_t>& s) {
+    for (std::map<mds_rank_t, mds_gid_t>::const_iterator p = up.begin();
 	 p != up.end();
 	 ++p)
       s.insert(p->first);
   }
-  void get_active_mds_set(set<int>& s) {
+  void get_active_mds_set(std::set<mds_rank_t>& s) {
     get_mds_set(s, MDSMap::STATE_ACTIVE);
   }
-  void get_failed_mds_set(set<int>& s) {
+  void get_failed_mds_set(std::set<mds_rank_t>& s) {
     s = failed;
   }
   int get_failed() {
     if (!failed.empty()) return *failed.begin();
     return -1;
   }
-  void get_stopped_mds_set(set<int>& s) {
+  void get_stopped_mds_set(std::set<mds_rank_t>& s) {
     s = stopped;
   }
-  void get_recovery_mds_set(set<int>& s) {
+  void get_recovery_mds_set(std::set<mds_rank_t>& s) {
     s = failed;
-    for (map<uint64_t,mds_info_t>::const_iterator p = mds_info.begin();
+    for (std::map<mds_gid_t, mds_info_t>::const_iterator p = mds_info.begin();
 	 p != mds_info.end();
 	 ++p)
       if (p->second.state >= STATE_REPLAY && p->second.state <= STATE_STOPPING)
 	s.insert(p->second.rank);
   }
-  void get_clientreplay_or_active_or_stopping_mds_set(set<int>& s) {
-    for (map<uint64_t,mds_info_t>::const_iterator p = mds_info.begin();
+  void get_clientreplay_or_active_or_stopping_mds_set(std::set<mds_rank_t>& s) {
+    for (std::map<mds_gid_t, mds_info_t>::const_iterator p = mds_info.begin();
 	 p != mds_info.end();
 	 ++p)
       if (p->second.state >= STATE_CLIENTREPLAY && p->second.state <= STATE_STOPPING)
 	s.insert(p->second.rank);
   }
-  void get_mds_set(set<int>& s, int state) {
-    for (map<uint64_t,mds_info_t>::const_iterator p = mds_info.begin();
+  void get_mds_set(std::set<mds_rank_t>& s, DaemonState state) {
+    for (std::map<mds_gid_t, mds_info_t>::const_iterator p = mds_info.begin();
 	 p != mds_info.end();
 	 ++p)
       if (p->second.state == state)
@@ -348,14 +380,14 @@ public:
   int get_random_up_mds() {
     if (up.empty())
       return -1;
-    map<int32_t,uint64_t>::iterator p = up.begin();
+    std::map<mds_rank_t, mds_gid_t>::iterator p = up.begin();
     for (int n = rand() % up.size(); n; n--)
       ++p;
     return p->first;
   }
 
-  const mds_info_t* find_by_name(const string& name) const {
-    for (map<uint64_t,mds_info_t>::const_iterator p = mds_info.begin();
+  const mds_info_t* find_by_name(const std::string& name) const {
+    for (std::map<mds_gid_t, mds_info_t>::const_iterator p = mds_info.begin();
 	 p != mds_info.end();
 	 ++p) {
       if (p->second.name == name)
@@ -364,10 +396,10 @@ public:
     return NULL;
   }
 
-  uint64_t find_standby_for(int mds, string& name) {
-    map<uint64_t, mds_info_t>::const_iterator generic_standby
+  mds_gid_t find_standby_for(mds_rank_t mds, std::string& name) {
+    std::map<mds_gid_t, mds_info_t>::const_iterator generic_standby
       = mds_info.end();
-    for (map<uint64_t,mds_info_t>::const_iterator p = mds_info.begin();
+    for (std::map<mds_gid_t, mds_info_t>::const_iterator p = mds_info.begin();
 	 p != mds_info.end();
 	 ++p) {
       if ((p->second.state != MDSMap::STATE_STANDBY && p->second.state != MDSMap::STATE_STANDBY_REPLAY) ||
@@ -381,81 +413,90 @@ public:
     }
     if (generic_standby != mds_info.end())
       return generic_standby->first;
-    return 0;
+    return MDS_GID_NONE;
   }
-  uint64_t find_unused_for(int mds, string& name) {
-    for (map<uint64_t,mds_info_t>::const_iterator p = mds_info.begin();
-	 p != mds_info.end();
-	 ++p) {
+
+  mds_gid_t find_unused_for(mds_rank_t mds, std::string& name) const {
+    for (std::map<mds_gid_t,mds_info_t>::const_iterator p = mds_info.begin();
+         p != mds_info.end();
+         ++p) {
       if (p->second.state != MDSMap::STATE_STANDBY ||
-	  p->second.laggy() ||
-	  p->second.rank >= 0)
-	continue;
+          p->second.laggy() ||
+          p->second.rank >= 0)
+        continue;
       if ((p->second.standby_for_rank == MDS_NO_STANDBY_PREF ||
-	   p->second.standby_for_rank == MDS_MATCHED_ACTIVE ||
-	   (p->second.standby_for_rank == MDS_STANDBY_ANY && g_conf->mon_force_standby_active))) {
-	return p->first;
+           p->second.standby_for_rank == MDS_MATCHED_ACTIVE ||
+           (p->second.standby_for_rank == MDS_STANDBY_ANY && g_conf->mon_force_standby_active))) {
+        return p->first;
       }
     }
-    return 0;
+    return MDS_GID_NONE;
   }
-  uint64_t find_replacement_for(int mds, string& name) {
-    uint64_t standby = find_standby_for(mds, name);
+
+  mds_gid_t find_replacement_for(mds_rank_t mds, std::string& name) {
+    const mds_gid_t standby = find_standby_for(mds, name);
     if (standby)
       return standby;
     else
       return find_unused_for(mds, name);
   }
 
-  void get_health(list<pair<health_status_t,string> >& summary,
-		  list<pair<health_status_t,string> > *detail) const;
+  void get_health(list<pair<health_status_t,std::string> >& summary,
+		  list<pair<health_status_t,std::string> > *detail) const;
 
   // mds states
-  bool is_down(int m) const { return up.count(m) == 0; }
-  bool is_up(int m) const { return up.count(m); }
-  bool is_in(int m) const { return up.count(m) || failed.count(m); }
-  bool is_out(int m) const { return !is_in(m); }
+  bool is_down(mds_rank_t m) const { return up.count(m) == 0; }
+  bool is_up(mds_rank_t m) const { return up.count(m); }
+  bool is_in(mds_rank_t m) const { return up.count(m) || failed.count(m); }
+  bool is_out(mds_rank_t m) const { return !is_in(m); }
 
-  bool is_failed(int m) const   { return failed.count(m); }
-  bool is_stopped(int m) const    { return stopped.count(m); }
+  bool is_failed(mds_rank_t m) const   { return failed.count(m); }
+  bool is_stopped(mds_rank_t m) const    { return stopped.count(m); }
 
-  bool is_dne(int m) const      { return in.count(m) == 0; }
-  bool is_dne_gid(uint64_t gid) const     { return mds_info.count(gid) == 0; }
+  bool is_dne(mds_rank_t m) const      { return in.count(m) == 0; }
+  bool is_dne_gid(mds_gid_t gid) const     { return mds_info.count(gid) == 0; }
 
-  int get_state(int m) const {
-    map<int32_t,uint64_t>::const_iterator u = up.find(m);
+  /**
+   * Get MDS rank state if the rank is up, else STATE_NULL
+   */
+  DaemonState get_state(mds_rank_t m) const {
+    std::map<mds_rank_t, mds_gid_t>::const_iterator u = up.find(m);
     if (u == up.end())
-      return 0;
+      return STATE_NULL;
     return get_state_gid(u->second);
   }
-  int get_state_gid(uint64_t gid) const {
-    map<uint64_t,mds_info_t>::const_iterator i = mds_info.find(gid);
+
+  /**
+   * Get MDS daemon status by GID
+   */
+  DaemonState get_state_gid(mds_gid_t gid) const {
+    std::map<mds_gid_t,mds_info_t>::const_iterator i = mds_info.find(gid);
     if (i == mds_info.end())
-      return 0;
+      return STATE_NULL;
     return i->second.state;
   }
 
-  mds_info_t& get_info(int m) { assert(up.count(m)); return mds_info[up[m]]; }
-  mds_info_t& get_info_gid(uint64_t gid) { assert(mds_info.count(gid)); return mds_info[gid]; }
+  mds_info_t& get_info(mds_rank_t m) { assert(up.count(m)); return mds_info[up[m]]; }
+  mds_info_t& get_info_gid(mds_gid_t gid) { assert(mds_info.count(gid)); return mds_info[gid]; }
 
-  bool is_boot(int m) const { return get_state(m) == STATE_BOOT; }
-  bool is_creating(int m) const { return get_state(m) == STATE_CREATING; }
-  bool is_starting(int m) const { return get_state(m) == STATE_STARTING; }
-  bool is_replay(int m) const   { return get_state(m) == STATE_REPLAY; }
-  bool is_resolve(int m) const  { return get_state(m) == STATE_RESOLVE; }
-  bool is_reconnect(int m) const { return get_state(m) == STATE_RECONNECT; }
-  bool is_rejoin(int m) const   { return get_state(m) == STATE_REJOIN; }
-  bool is_clientreplay(int m) const { return get_state(m) == STATE_CLIENTREPLAY; }
-  bool is_active(int m) const  { return get_state(m) == STATE_ACTIVE; }
-  bool is_stopping(int m) const { return get_state(m) == STATE_STOPPING; }
-  bool is_active_or_stopping(int m) const {
+  bool is_boot(mds_rank_t m) const { return get_state(m) == STATE_BOOT; }
+  bool is_creating(mds_rank_t m) const { return get_state(m) == STATE_CREATING; }
+  bool is_starting(mds_rank_t m) const { return get_state(m) == STATE_STARTING; }
+  bool is_replay(mds_rank_t m) const   { return get_state(m) == STATE_REPLAY; }
+  bool is_resolve(mds_rank_t m) const  { return get_state(m) == STATE_RESOLVE; }
+  bool is_reconnect(mds_rank_t m) const { return get_state(m) == STATE_RECONNECT; }
+  bool is_rejoin(mds_rank_t m) const   { return get_state(m) == STATE_REJOIN; }
+  bool is_clientreplay(mds_rank_t m) const { return get_state(m) == STATE_CLIENTREPLAY; }
+  bool is_active(mds_rank_t m) const  { return get_state(m) == STATE_ACTIVE; }
+  bool is_stopping(mds_rank_t m) const { return get_state(m) == STATE_STOPPING; }
+  bool is_active_or_stopping(mds_rank_t m) const {
     return is_active(m) || is_stopping(m);
   }
-  bool is_clientreplay_or_active_or_stopping(int m) const {
+  bool is_clientreplay_or_active_or_stopping(mds_rank_t m) const {
     return is_clientreplay(m) || is_active(m) || is_stopping(m);
   }
 
-  bool is_followable(int m) const {
+  bool is_followable(mds_rank_t m) const {
     return (is_resolve(m) ||
 	    is_replay(m) ||
 	    is_rejoin(m) ||
@@ -464,22 +505,22 @@ public:
 	    is_stopping(m));
   }
 
-  bool is_laggy_gid(uint64_t gid) const {
+  bool is_laggy_gid(mds_gid_t gid) const {
     if (!mds_info.count(gid))
       return false;
-    map<uint64_t,mds_info_t>::const_iterator p = mds_info.find(gid);
+    std::map<mds_gid_t,mds_info_t>::const_iterator p = mds_info.find(gid);
     return p->second.laggy();
   }
 
 
   // cluster states
   bool is_full() const {
-    return in.size() >= max_mds;
+    return mds_rank_t(in.size()) >= max_mds;
   }
   bool is_degraded() const {   // degraded = some recovery in process.  fixes active membership and recovery_set.
     if (!failed.empty())
       return true;
-    for (map<uint64_t,mds_info_t>::const_iterator p = mds_info.begin();
+    for (std::map<mds_gid_t,mds_info_t>::const_iterator p = mds_info.begin();
 	 p != mds_info.end();
 	 ++p)
       if (p->second.state >= STATE_REPLAY && p->second.state <= STATE_CLIENTREPLAY)
@@ -509,18 +550,18 @@ public:
   }
 
   // inst
-  bool have_inst(int m) {
+  bool have_inst(mds_rank_t m) {
     return up.count(m);
   }
-  const entity_inst_t get_inst(int m) {
+  const entity_inst_t get_inst(mds_rank_t m) {
     assert(up.count(m));
     return mds_info[up[m]].get_inst();
   }
-  const entity_addr_t get_addr(int m) {
+  const entity_addr_t get_addr(mds_rank_t m) {
     assert(up.count(m));
     return mds_info[up[m]].addr;
   }
-  bool get_inst(int m, entity_inst_t& inst) { 
+  bool get_inst(mds_rank_t m, entity_inst_t& inst) {
     if (up.count(m)) {
       inst = get_inst(m);
       return true;
@@ -528,18 +569,18 @@ public:
     return false;
   }
   
-  int get_rank_gid(uint64_t gid) {
+  mds_rank_t get_rank_gid(mds_gid_t gid) {
     if (mds_info.count(gid))
       return mds_info[gid].rank;
-    return -1;
+    return MDS_RANK_NONE;
   }
 
-  int get_inc(int m) {
+  int get_inc(mds_rank_t m) {
     if (up.count(m)) 
       return mds_info[up[m]].inc;
     return 0;
   }
-  int get_inc_gid(uint64_t gid) {
+  int get_inc_gid(mds_gid_t gid) {
     if (mds_info.count(gid))
       return mds_info[gid].inc;
     return -1;
